<?php

declare(strict_types=1);

namespace Drupal\ui_patterns_layouts\Plugin\Layout;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\Attribute\Layout;
use Drupal\Core\Layout\LayoutDefault;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\Context\Context;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\Plugin\Context\EntityContext;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\ui_patterns\ContextHelperInterface;
use Drupal\ui_patterns\Form\ComponentFormBuilderTrait;
use Drupal\ui_patterns\SourcePluginBase;
use Drupal\ui_patterns_layouts\Plugin\Derivative\ComponentLayout as DerivativeComponentLayout;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Component Layout.
 */
#[Layout(
  id: "ui_patterns",
  deriver: DerivativeComponentLayout::class
)]
class ComponentLayout extends LayoutDefault implements ContainerFactoryPluginInterface, PluginFormInterface {

  use ComponentFormBuilderTrait;

  /**
   * Constructs a new Component Layout instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\ui_patterns\ContextHelperInterface $contextHelper
   *   The context helper.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected ContextHelperInterface $contextHelper,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $plugin = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('ui_patterns.context_helper')
    );
    return $plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + $this->getComponentFormDefault();
  }

  /**
   * Returns the region names.
   *
   * @return string[]
   *   The region names.
   */
  public function getRegionNames() {
    return $this->pluginDefinition->getRegionNames();
  }

  /**
   * Returns the region.
   *
   * @return array[]
   *   The regions.
   */
  public function getRegions() {
    return $this->pluginDefinition->getRegions();
  }

  /**
   * Get the entity context if exists.
   *
   * @return array
   *   Source contexts.
   */
  protected function getComponentSourceContexts(FormStateInterface $form_state = NULL): array {
    $gatheredContexts = $form_state ? $form_state->getTemporaryValue('gathered_contexts') : [];
    $this->contextHelper->applyContextMapping($this, $gatheredContexts ?: []);
    if (!isset($this->context['entity']) || !($this->context['entity']->getContextValue() instanceof EntityInterface)) {
      if ($entity = $this->contextHelper->guessEntity($this->context, $form_state)) {
        $this->context['entity'] = EntityContext::fromEntity($entity);
      }
    }
    if (!isset($this->context["bundle"]) && isset($this->context["entity"]) && ($entity = $this->context["entity"]->getContextValue())) {
      $this->context['bundle'] = new Context(ContextDefinition::create('string'), $entity->bundle() ?? "");
    }
    return $this->context;
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $regions) {
    $build = $this->buildComponentRenderable($this->getPluginDefinition()->id(), $this->getComponentSourceContexts());
    $build['#layout'] = $this;
    $regions = parent::build($regions);
    foreach ($this->getPluginDefinition()->getRegionNames() as $region_name) {
      if (!isset($regions[$region_name])) {
        continue;
      }
      $build['#slots'][$region_name] = $regions[$region_name];
      // For layout builder we need to pass slots also
      // direct to the build array. They are needed to generate contextual
      // links and other surrounding markup.
      $build[$region_name] = $regions[$region_name];
    }
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $contexts = $this->getComponentSourceContexts($form_state);
    $form = parent::buildConfigurationForm($form, $form_state);
    return $form + $this->buildComponentsForm($form_state, $contexts, $this->getPluginDefinition()->id(), FALSE, TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) : void {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state):void {
    $this->submitComponentsForm($form_state);
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();
    $component_id = $this->getPluginDefinition()->id();
    // Why no getComponentSourceContexts?
    $component_dependencies = $this->calculateComponentDependencies($component_id);
    SourcePluginBase::mergeConfigDependencies($dependencies, $component_dependencies);
    SourcePluginBase::mergeConfigDependencies($dependencies, ["module" => ["ui_patterns_layouts"]]);
    return $dependencies;
  }

}
