<?php

declare(strict_types=1);

namespace Drupal\ui_patterns;

use Drupal\Component\Plugin\Exception\ContextException;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\Context\ContextInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\Plugin\Context\EntityContext;
use Drupal\Core\Plugin\ContextAwarePluginInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\field_layout\Entity\FieldLayoutEntityFormDisplay;
use Drupal\field_layout\Entity\FieldLayoutEntityViewDisplay;
use Drupal\field_layout\Form\FieldLayoutEntityFormDisplayEditForm;
use Drupal\field_layout\Form\FieldLayoutEntityViewDisplayEditForm;
use Drupal\layout_builder\DefaultsSectionStorageInterface;
use Drupal\layout_builder\Form\ConfigureSectionForm;
use Drupal\layout_builder\OverridesSectionStorageInterface;
use Drupal\layout_builder\SectionStorageInterface;
use Drupal\ui_patterns\Entity\SampleEntityGeneratorInterface;

/**
 * Context helper used by drupal plugins to get the context.
 */
class ContextHelper implements ContextHelperInterface {

  public function __construct(
    protected SampleEntityGeneratorInterface $sampleEntityGenerator,
    protected ContextRepositoryInterface $contextRepository,
    protected ContextHandlerInterface $contextHandler,
    protected ModuleHandlerInterface $moduleHandler,
    protected RouteMatchInterface $routeMatch,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityFieldManagerInterface $entityFieldManager,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public function guessEntity(array $contexts = [], FormStateInterface $form_state = NULL): ?EntityInterface {
    if ($this->moduleHandler->moduleExists("layout_builder")) {
      // Try to get the $section storage.
      $section_storage = $this->getLayoutBuilderSectionStorage($form_state);
      if ($section_storage && ($entity = $this->guessLayoutBuilderEntity($section_storage))) {
        return $entity;
      }
    }
    if ($this->moduleHandler->moduleExists("field_layout")) {
      if ($entity = $this->guessFieldLayoutEntity($form_state)) {
        return $entity;
      }
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getSampleEntity(string $entity_type_id, string $bundle) : EntityInterface {
    return $this->sampleEntityGenerator->get($entity_type_id, $bundle);
  }

  /**
   * {@inheritdoc}
   */
  public function getSampleEntityWithField(string $entity_type_id, string $field_name) : EntityInterface {
    return $this->getSampleEntity($entity_type_id, $this->findEntityBundleWithField($entity_type_id, $field_name));
  }

  /**
   * {@inheritdoc}
   */
  public function checkEntityHasField(EntityInterface $entity, string $entity_type_id, string $field_name) : bool {
    $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
    return ($entity->getEntityTypeId() === $entity_type_id &&
      array_key_exists($field_name, $field_definitions));
  }

  /**
   * Find an entity bundle which has a field.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param string $field_name
   *   The field name to be found in searched bundle.
   *
   * @return string
   *   The bundle.
   */
  public function findEntityBundleWithField(string $entity_type_id, string $field_name) : string {
    // @todo better implementation with service 'entity_type.bundle.info'
    $bundle = $entity_type_id;
    $bundle_entity_type = $this->entityTypeManager->getDefinition($entity_type_id)->getBundleEntityType();
    if (NULL !== $bundle_entity_type) {
      $bundle_list = $this->entityTypeManager->getStorage($bundle_entity_type)->loadMultiple();
      if (count($bundle_list) > 0) {
        foreach ($bundle_list as $bundle_entity) {
          $bundle_to_test = (string) $bundle_entity->id();
          $definitions = $this->entityFieldManager->getFieldDefinitions($entity_type_id, $bundle_to_test);
          if (array_key_exists($field_name, $definitions)) {
            $bundle = $bundle_to_test;
            break;
          }
        }
      }
    }
    return $bundle;
  }

  /**
   * {@inheritdoc}
   */
  public function applyContextMapping(ContextAwarePluginInterface $plugin, array $gathered_contexts = []) : void {
    $available_contexts = $this->contextRepository->getAvailableContexts();
    if (!is_array($gathered_contexts)) {
      $gathered_contexts = [];
    }
    if (!empty($gathered_contexts)) {
      $gathered_contexts = array_filter($gathered_contexts, fn($context) => $context instanceof ContextInterface);
      $available_contexts = array_merge($available_contexts, $gathered_contexts);
    }
    try {
      $this->contextHandler->applyContextMapping($plugin, $available_contexts);
    }
    catch (ContextException $e) {
      // Do nothing.
    }
  }

  /**
   * Gets the layout builder section storage.
   *
   * @param \Drupal\Core\Form\FormStateInterface|null $form_state
   *   Optional Form state.
   *
   * @return \Drupal\layout_builder\SectionStorageInterface|null
   *   The section storage or null.
   */
  protected function getLayoutBuilderSectionStorage(FormStateInterface $form_state = NULL) : ?SectionStorageInterface {
    if ($form_state !== NULL) {
      $form_object = $form_state->getFormObject();
      if ($form_object instanceof ConfigureSectionForm) {
        return $form_object->getSectionStorage();
      }
    }
    $section_storage = $this->routeMatch->getParameter("section_storage");
    if ($section_storage instanceof SectionStorageInterface) {
      return $section_storage;
    }
    return NULL;
  }

  /**
   * Gets entity information from sections storage.
   *
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section to look up info on.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity if found or null.
   */
  protected function guessLayoutBuilderEntity(SectionStorageInterface $section_storage) : ?EntityInterface {
    if ($entity = $this->guessLayoutBuilderEntityFromContexts($section_storage->getContexts())) {
      return $entity;
    }
    $storage_id = $section_storage->getStorageId();
    if ($section_storage instanceof DefaultsSectionStorageInterface) {
      $display = $this->entityTypeManager->getStorage('entity_view_display')->load($storage_id);
      if ($display instanceof EntityViewDisplayInterface) {
        return $this->sampleEntityGenerator->get($display->getTargetEntityTypeId(), $display->getTargetBundle());
      }
    }
    elseif ($section_storage instanceof OverridesSectionStorageInterface) {
      [$entity_type_id, $id] = explode('.', $storage_id);
      if ($entity = $this->entityTypeManager->getStorage($entity_type_id)->load($id)) {
        return $entity;
      }
    }
    return NULL;
  }

  /**
   * Gets entity information from sections storage.
   *
   * @param array<mixed> $section_storage_contexts
   *   Contexts.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity if found or null.
   */
  protected function guessLayoutBuilderEntityFromContexts(array $section_storage_contexts) : ?EntityInterface {
    if (array_key_exists("entity", $section_storage_contexts) && ($entity = $section_storage_contexts["entity"]->getContextValue())) {
      return $entity;
    }
    // Search for the entity in the context (no matter the name).
    foreach ($section_storage_contexts as $section_storage_context) {
      if ($section_storage_context instanceof EntityContext) {
        // Now check if it is an entity view display.
        $entity_in_context = $section_storage_context->getContextValue();
        if ($entity_in_context instanceof EntityViewDisplayInterface) {
          return $this->sampleEntityGenerator->get($entity_in_context->getTargetEntityTypeId(), $entity_in_context->getTargetBundle());
        }
      }
    }
    return NULL;
  }

  /**
   * Guess the entity from the field layout form.
   *
   * @param \Drupal\Core\Form\FormStateInterface|null $form_state
   *   Optional Form state.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity if found or null.
   */
  protected function guessFieldLayoutEntity(?FormStateInterface $form_state = NULL) : ?EntityInterface {
    if ($form_state !== NULL) {
      $form_object = $form_state->getFormObject();
      if (($form_object instanceof FieldLayoutEntityViewDisplayEditForm) ||
        ($form_object instanceof FieldLayoutEntityFormDisplayEditForm)) {
        $entity = $form_object->getEntity();
        if (($entity instanceof FieldLayoutEntityViewDisplay)|| ($entity instanceof FieldLayoutEntityFormDisplay)) {
          return $this->getSampleEntity($entity->getTargetEntityTypeId(), $entity->getTargetBundle());
        }
      }
    }
    return NULL;
  }

}
