<?php

declare(strict_types=1);

namespace Drupal\ui_patterns;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContextAwarePluginInterface;

/**
 * Interface for Context helper used by drupal plugins.
 */
interface ContextHelperInterface {

  /**
   * Get an entity from contexts.
   *
   * @param array<mixed> $contexts
   *   Known contexts.
   * @param \Drupal\Core\Form\FormStateInterface|null $form_state
   *   Optional Form state.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   Entity or null.
   */
  public function guessEntity(array $contexts = [], FormStateInterface $form_state = NULL): ?EntityInterface;

  /**
   * Check if the entity is compatible with the field.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param string $entity_type_id
   *   The entity type id.
   * @param string $field_name
   *   The field name.
   *
   * @return bool
   *   TRUE if the entity is compatible.
   */
  public function checkEntityHasField(EntityInterface $entity, string $entity_type_id, string $field_name) : bool;

  /**
   * Return an entity of the desired type.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param string $bundle
   *   The bundle.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The entity.
   */
  public function getSampleEntity(string $entity_type_id, string $bundle) : EntityInterface;

  /**
   * Get an entity with the desired field.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param string $field_name
   *   The field name.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The entity with the field.
   */
  public function getSampleEntityWithField(string $entity_type_id, string $field_name) : EntityInterface;

  /**
   * Apply context mapping to a plugin.
   *
   * @param \Drupal\Core\Plugin\ContextAwarePluginInterface $plugin
   *   The plugin to apply context mapping to.
   * @param array $gathered_contexts
   *   Contexts to use in addition to contexts from contextRepository.
   */
  public function applyContextMapping(ContextAwarePluginInterface $plugin, array $gathered_contexts = []) : void;

}
