<?php

namespace Drupal\ui_patterns\Element;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\ui_patterns\SourceInterface;
use Drupal\ui_patterns\SourcePluginBase;

/**
 * Component to render a single prop.
 *
 * Usage example:
 *
 * @code
 * $form['prop_name'] = [
 *   '#type' => 'component_prop_form',
 *   '#component_id' => 'component_id',
 *   '#prop_id' => 'prop'
 *   '#default_value' => [
 *     'source' => [],
 *     'source_id' => 'textfield'
 *   ],
 * ];
 * @endcode
 *
 * Value example:
 *
 * @code
 * '#default_value' => ['source_id' => 'id', 'source' => []]
 * @endcode
 *
 *  Configuration:
 *
 *  '#component_id' => Required Component ID.
 *  '#prop_id' => Required Prop ID.
 *  '#source_contexts' => The context of the sources.
 *  '#tag_filter' => Filter sources based on these tags.
 *
 * @FormElement("component_prop_form")
 */
class ComponentPropForm extends ComponentFormBase {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#input' => TRUE,
      '#multiple' => FALSE,
      '#default_value' => NULL,
      '#source_contexts' => [],
      '#tag_filter' => [],
      '#component_id' => NULL,
      '#slot_id' => NULL,
      '#process' => [
        [$class, 'buildForm'],
      ],
      '#theme_wrappers' => ['fieldset'],
    ];
  }

  /**
   * Build props forms.
   */
  public static function buildForm(array &$element, FormStateInterface $form_state): array {
    $element['#tree'] = TRUE;
    $prop_id = $element['#prop_id'];
    $component = static::getComponent($element);
    $props = $component->metadata->schema['properties'];
    $definition = $props[$prop_id];
    $configuration = $element['#default_value'] ?? [];
    $sources = static::getSources($prop_id, $definition, $element);
    $selected_source = static::getSelectedSource($configuration, $sources);
    if (!$selected_source) {
      $selected_source = static::getDefaultSource($prop_id, $definition, $element);
      if ($selected_source && !isset($sources[$selected_source->getPluginId()])) {
        $selected_source = current($sources);
      }
    }
    if (!$selected_source) {
      return [];
    }

    $wrapper_id = static::getElementId($element, 'ui-patterns-prop-item-' . $prop_id);
    $source_selector = static::buildSourceSelector($sources, $selected_source, $wrapper_id);
    $source_form = static::getSourcePluginForm($form_state, $selected_source, $wrapper_id);

    $element += [
      'source_id' => $source_selector,
      'source' => $source_form,
    ];
    $element['#attributes']['style'] = 'position: relative;';
    $element = static::addRequired($element, $prop_id);
    return $element;
  }

  /**
   * Add required visual clue to the fieldset.
   *
   * The proper required control is managed by SourcePluginBase::addRequired()
   * so the visual clue is present whether or not the control is done by the
   * source plugin. This is feature, not a bug.
   */
  protected static function addRequired(array $element, string $prop_id): array {
    $component = static::getComponent($element);
    if (!isset($component->metadata->schema["required"])) {
      return $element;
    }
    $required_props = $component->metadata->schema["required"];
    if (!in_array($prop_id, $required_props)) {
      return $element;
    }
    $element["#required"] = TRUE;
    return $element;
  }

  /**
   * Get source plugin form.
   */
  protected static function getSourcePluginForm(FormStateInterface $form_state, SourceInterface $source, string $wrapper_id): array {
    $form = $source->settingsForm([], $form_state);
    $form["#type"] = 'container';
    $form['#attributes'] = [
      'id' => $wrapper_id,
    ];
    // Weird, but :switchSourceForm() AJAX handler doesn't work without that.
    foreach (Element::children($form) as $child) {
      if (isset($form[$child]['#description']) && !isset($form[$child]['#description_display'])) {
        $form[$child]['#description_display'] = 'after';
      }
    }
    return $form;
  }

  /**
   * Build sources selector widget.
   */
  protected static function buildSourceSelector(array $sources, SourceInterface $selected_source, string $wrapper_id): array {
    if (empty($sources)) {
      return [];
    }
    if (count($sources) == 1) {
      return [
        '#type' => 'hidden',
        '#value' => array_keys($sources)[0],
      ];
    }
    $options = [];
    foreach ($sources as $source_id => $source) {
      $options[$source_id] = $source->label();
    }
    return [
      '#type' => 'select',
      "#options" => $options,
      '#default_value' => $selected_source->getPluginId(),
      '#attributes' => [
        'style' => "position: absolute; top: 0; right: 0;",
      ],
      '#prop_id' => $selected_source->getPropId(),
      '#prop_definition' => $selected_source->getPropDefinition(),
      '#ajax' => [
        'callback' => [
          static::class,
          'switchSourceForm',
        ],
        'wrapper' => $wrapper_id,
        'effect' => 'fade',
      ],
    ];
  }

  /**
   * Ajax handler: Switch source plugin form.
   */
  public static function switchSourceForm(array $form, FormStateInterface $form_state): array {
    $selector = $form_state->getTriggeringElement();
    $parents = $selector["#array_parents"];
    $subform = NestedArray::getValue($form, array_slice($parents, 0, -1));
    return $subform["source"];
  }

  /**
   * Get sources for a prop type.
   */
  protected static function getSources(string $prop_id, array $definition, array $element): array {
    $configuration = $element['#default_value'] ?? [];
    $source_contexts = $element['#source_contexts'];
    $form_array_parents = $element['#array_parents'];
    $tag_filter = $element['#tag_filter'];
    $prop_type = $definition['ui_patterns']['type_definition'];
    /** @var \Drupal\ui_patterns\SourcePluginManager $source_plugin_manager */
    $source_plugin_manager = \Drupal::service("plugin.manager.ui_patterns_source");
    $source_ids = array_keys($source_plugin_manager->getDefinitionsForPropType($prop_type->getPluginId(), $source_contexts, $tag_filter));
    $source_ids = array_combine($source_ids, $source_ids);
    if (empty($source_ids)) {
      return [];
    }
    return $source_plugin_manager->createInstances($source_ids, SourcePluginBase::buildConfiguration($prop_id, $definition, $configuration, $source_contexts, $form_array_parents));
  }

  /**
   * Get selected source plugin.
   */
  protected static function getSelectedSource(array $configuration, array $sources): ?SourceInterface {
    if (isset($configuration['source_id']) && $sources[$configuration['source_id']]) {
      return $sources[$configuration['source_id']];
    }
    return NULL;
  }

  /**
   * Get default source plugin.
   *
   * @param string $prop_id
   *   The prop ID.
   * @param array $definition
   *   The prop definition.
   * @param array $element
   *   The form element.
   *
   * @return \Drupal\ui_patterns\SourceInterface|null
   *   The source plugin or NULL.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected static function getDefaultSource(string $prop_id, array $definition, array $element): ?SourceInterface {
    $configuration = $element['#default_value'] ?? [];
    $source_contexts = $element['#source_contexts'];
    $form_array_parents = $element['#array_parents'];
    $tag_filter = $element['#tag_filter'];
    /** @var \Drupal\ui_patterns\SourcePluginManager $source_plugin_manager */
    $source_plugin_manager = \Drupal::service("plugin.manager.ui_patterns_source");
    $prop_type = $definition['ui_patterns']['type_definition'];
    $source_id = $source_plugin_manager->getPropTypeDefault($prop_type->getPluginId(), $source_contexts, $tag_filter);
    if (!$source_id) {
      return NULL;
    }
    $plugin_configuration = SourcePluginBase::buildConfiguration($prop_id, $definition, $configuration, $source_contexts, $form_array_parents);
    /** @var \Drupal\ui_patterns\SourceInterface */
    return $source_plugin_manager->createInstance($source_id, $plugin_configuration);
  }

}
