<?php

declare(strict_types=1);

namespace Drupal\ui_patterns\Plugin\UiPatterns\Source;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ui_patterns\Attribute\Source;
use Drupal\ui_patterns\SourcePluginBase;

/**
 * Plugin implementation of the source.
 */
#[Source(
  id: 'attributes',
  label: new TranslatableMarkup('Attributes'),
  description: new TranslatableMarkup('...'),
  prop_types: ['attributes']
)]
class AttributesWidget extends SourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function getPropValue(): mixed {
    // In UI Patterns Settings, we built the Attribute object here. It is not
    // possible anymore because SDC will not validate it against the prop
    // type schema.
    $value = $this->getSetting('value');
    if (!is_string($value)) {
      return [];
    }
    return $this->convertStringToAttributesMapping($value);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = parent::settingsForm($form, $form_state);
    // Attributes are associative arrays, but this source plugin is storing
    // them as string in config.
    // It would be better to use something else than a textfield one day.
    $form['value'] = [
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('value'),
    ];
    // Allow anything in attributes values, which are between simple and double
    // quotes. Forbid some characters in attributes names.
    // See https://www.w3.org/TR/2011/WD-html5-20110525/syntax.html#attributes-0
    $forbidden_characters = "<>&/`";
    $pattern = "^(([^" . $forbidden_characters . "]+)=[\"'].+[\"']\s*)*?$";
    $form['value']['#pattern'] = $pattern;
    $form['value']['#description'] = $this->t("Values must be present and quoted.");
    $this->addRequired($form['value']);
    return $form;
  }

  /**
   * Convert a string to an attribute mapping.
   */
  protected function convertStringToAttributesMapping(string $value): array {
    $parse_html = '<div ' . $value . '></div>';
    $attributes = [];
    foreach (Html::load($parse_html)->getElementsByTagName('div') as $div) {
      foreach ($div->attributes as $attr) {
        $attributes[$attr->nodeName] = $attr->nodeValue;
      }
    }
    return $attributes;
  }

}
